package inputport.datacomm.duplex.buffer;

import global.GlobalState;
import inputport.datacomm.duplex.NoMessageReceivedByResponderException;
import inputport.datacomm.simplex.SendTrapper;
import inputport.datacomm.simplex.buffer.AGenericSimplexBufferServerInputPort;
import inputport.datacomm.simplex.buffer.ASendRegistrarAndNotifier;
import inputport.datacomm.simplex.buffer.ByteBufferSendListener;
import inputport.datacomm.simplex.buffer.SendRegistrarAndNotifier;

import java.nio.ByteBuffer;

import common.Tracer;
public class AGeneriDuplexBufferServerInputPort<RequestChannelType, MessageChannelType> extends AGenericSimplexBufferServerInputPort<RequestChannelType, MessageChannelType> 
             implements BufferGenericDuplexServerInputPort<RequestChannelType, MessageChannelType> {
	SendTrapper<ByteBuffer, ByteBuffer> sendTrapper;
	protected String lastSender = null;
	SendRegistrarAndNotifier sendRegistrarAndNotifier = new ASendRegistrarAndNotifier();
	
	DuplexServerInputPortDriver<RequestChannelType, MessageChannelType> duplexDriver;

	public SendTrapper<ByteBuffer, ByteBuffer> getSendTrapper() {
		return sendTrapper;
	}
	public void setSendTrapper(SendTrapper<ByteBuffer, ByteBuffer> aSendTrapper) {
		Tracer.info(this, "Setting my send trapper:" + aSendTrapper);
		this.sendTrapper = aSendTrapper;
	}
	public AGeneriDuplexBufferServerInputPort( String theMyName, String anId) {
		super(theMyName, anId);
	}
	@Override
	public void setDriver(DuplexServerInputPortDriver<RequestChannelType, MessageChannelType> theDriver) {
		super.setDriver(theDriver);
		duplexDriver = theDriver;
		sendTrapper = GlobalState.getServerBufferDuplexIPTrapperSelector().createSendTrapper(this, theDriver);
		Tracer.info(this, "Retrieved from global state send trapper:" + sendTrapper );
	}	

	@Override
	public void reply(ByteBuffer message) {
		if (getLastSender() == null) throw new NoMessageReceivedByResponderException();
		Tracer.info(this, "Replying:" + message);
		send(getLastSender(), message);
	}
	@Override
	public String getLastSender() {
		return lastSender;
	}
	@Override
	public void setLastSender(String newVal) {
		if (newVal.equals(lastSender)) return;
		Tracer.info(this, "Setting reply receiver:" + lastSender);
		lastSender = newVal;
	}

	@Override
	public void send(String aRemoteName, ByteBuffer aMessage) {
		Tracer.info(this, "Sending driver message:" + aMessage);
		sendTrapper.send(aRemoteName, aMessage);

	}
	@Override
	public void notifyPortReceive (String remoteEnd, ByteBuffer message) {
		setLastSender(remoteEnd);
		super.notifyPortReceive(remoteEnd, message);
	}
	
	public void addSendListener(ByteBufferSendListener portSendListener) {
		sendRegistrarAndNotifier.addSendListener(portSendListener);
	}
	public void notifyPortSend(String aRemoteEnd, ByteBuffer message, int sendId) {
		sendRegistrarAndNotifier.notifyPortSend(aRemoteEnd, message, sendId);
	}
	@Override
	public void messageSent(String aRemoteEnd, ByteBuffer aMessage, int aSendId) {
		Tracer.info(this, "Received sent message notification from driver");
		notifyPortSend(aRemoteEnd, aMessage, aSendId);
	}
	public void removeSendListener(ByteBufferSendListener portSendListener) {
		sendRegistrarAndNotifier.removeSendListener(portSendListener);
	}
	@Override
	public void send(ByteBuffer message) {
		reply(message);
		
	}
		

	

	

}
