package inputport.datacomm.duplex.buffer;

import global.GlobalState;
import inputport.datacomm.simplex.AReceiveRegistrarAndNotifier;
import inputport.datacomm.simplex.ReceiveListener;
import inputport.datacomm.simplex.ReceiveRegistrarAndNotifier;
import inputport.datacomm.simplex.ReceiveTrapper;
import inputport.datacomm.simplex.buffer.AGenericSimplexBufferClientInputPort;

import java.nio.ByteBuffer;

import common.Tracer;


public class AGenericDuplexBufferClientInputPort<ChannelType> extends AGenericSimplexBufferClientInputPort<ChannelType> implements  BufferGenericDuplexClientInputPort<ChannelType>{
	
	ReceiveRegistrarAndNotifier aNotifier = new AReceiveRegistrarAndNotifier();
	ReceiveTrapper<ByteBuffer, ByteBuffer> receiveTrapper;
	String lastSender;
	public AGenericDuplexBufferClientInputPort( String aRemoteHostName, String aRemotePort, String aRemoteName, String aMyName) {
		super(aRemoteHostName, aRemotePort, aRemoteName, aMyName);
		receiveTrapper = GlobalState.getClientBufferDuplexIPTrapperSelector().createReceiveTrapper(this, aNotifier);
		Tracer.info(this, "Retrieved from global state receive trapper:" + receiveTrapper);
	}	
	public void addReceiveListener(ReceiveListener<ByteBuffer> portReceiveListener) {
		aNotifier.addReceiveListener(portReceiveListener);
	}

	public void removeReceiveListener(ReceiveListener<ByteBuffer> portReceiveListener) {
		aNotifier.removeReceiveListener(portReceiveListener);
	}
	public void notifyPortReceive(String remoteEnd, ByteBuffer message) {
		receiveTrapper.notifyPortReceive(remoteEnd, message);
	}

	@Override
	public void messageReceived(String aSourceName, ByteBuffer aMessage) {
		Tracer.info(this, "Received message from driver");
		notifyPortReceive(aSourceName, aMessage);
	}
	
	@Override
	public void reply(ByteBuffer message) {
		send(message);
	}
	
	
	public ReceiveTrapper<ByteBuffer, ByteBuffer> getReceiveTrapper() {
		return receiveTrapper;
	}

	public void setReceiveTrapper(ReceiveTrapper<ByteBuffer, ByteBuffer> aReceiveTrapper) {
		Tracer.info(this, "Set my receive trapper to:" + aReceiveTrapper);
		this.receiveTrapper = aReceiveTrapper;
	}
	
	@Override
	public String getLastSender() {
		return getRemoteEndPoint();
	}
	


	@Override
	public void setLastSender(String newVal) {
		Tracer.info(this, "Surprinsingly, setting client last sender to:" + newVal);
			lastSender = newVal;
	}
	
	

}
