package inputport.datacomm.simplex.buffer;

import global.GlobalState;
import inputport.AConnectRegistrarAndNotifier;
import inputport.ConnectionListener;
import inputport.ConnectionRegistrarAndNotifier;
import inputport.datacomm.simplex.SendToUnconnectedPortException;
import inputport.datacomm.simplex.SendTrapper;

import java.nio.ByteBuffer;
import java.util.HashSet;
import java.util.Set;

import common.Tracer;

public class AGenericSimplexBufferClientInputPort<ChannelType>  implements GenericSimplexClientInputPort<ChannelType>  {
	
	ConnectionRegistrarAndNotifier connectNotifier = new AConnectRegistrarAndNotifier();
	protected SendTrapper<ByteBuffer, ByteBuffer> sendTrapper;
	
	SendRegistrarAndNotifier sendNotfier = new ASendRegistrarAndNotifier();	
	String myName;
	protected String serverDescription, serverName;
	protected Set<String> serverNames;
	protected Set<String> emptySet = new HashSet();

	boolean connected = false;
	String[] remoteNames;
	SimplexBufferClientInputPortDriver<ChannelType> driver;

	public AGenericSimplexBufferClientInputPort( String theRemoteHostName, String theRemotePort, String aRemoteName, String theMyName) {

		if (theMyName != null)
			myName = theMyName;
		else {
			myName = "Random Name:" + Math.random();
		}
		serverDescription = theRemoteHostName+":"+theRemotePort;
		serverName = aRemoteName;
		serverNames = new HashSet();
		serverNames.add(serverName);
	}
	@Override
	public void setDriver(SimplexBufferClientInputPortDriver<ChannelType> aChannelImplementation) {
		driver = aChannelImplementation;	
		sendTrapper = GlobalState.getSimplexBufferClientIPTrapperSelector().createSendTrapper(this, aChannelImplementation);
		Tracer.info(this, "Retrieved from global state send trapper:" + sendTrapper);
		setSendTrapper(sendTrapper);
	}
	@Override
	public String getRemoteEndPoint() {
		return serverName;
	}
	@Override
	public String getLocalName() {
		return myName;
	}

	@Override
	public void connect() {
		if (connected) {
			Tracer.info(this, "Ignoring connect call as already connected");
			return;
		}
		Tracer.info(this, "Asking driver to connect and changing status");
		connected = true;
		driver.connect();
	}
	void sendClientName() {
		ByteBuffer message =  ByteBuffer.wrap (myName.getBytes());
		Tracer.info(this, "Sending to server my name: " +  myName);
		send(message);
	}	
	@Override
	public void send(ByteBuffer message) {
		send (serverName, message);		

	}

	@Override
	public void send(String remoteName, ByteBuffer message) {
		if (!connected) throw new SendToUnconnectedPortException();
		Tracer.info(this, "Forwarding message to send trapper:" + sendTrapper);
		sendTrapper.send(remoteName, message);		
	}
	@Override
	public void disconnect() {
		Tracer.info(this, "Disconnecting driver and notifying listeners");
		driver.disconnect();
		connected = false;	
		notifyDisconnect(serverName, true, "Local process closed connection to remote end");
	}

	public Set<String> getConnections() {
		if (connected)
			return serverNames;
		else
			return emptySet;
	}
	public void addConnectionListener(ConnectionListener portConnectListener) {
		connectNotifier.addConnectionListener(portConnectListener);
	}
	public void removeConnectionListener(ConnectionListener portConnectListener) {
		connectNotifier.removeConnectionListener(portConnectListener);
	}
	public void notifyDisconnect(String aRemoteEnd, boolean anExplicitClose,
			String anExplanation) {
		connectNotifier.notifyDisconnect(aRemoteEnd, anExplicitClose, anExplanation);
	}	
	public void notifyConnectFailure(String aRemoteEnd, String anExplanation) {
		connectNotifier.notifyConnectFailure(aRemoteEnd, anExplanation);
	}
	@Override
	public void notifyConnect(String remoteEnd) {
		connectNotifier.notifyConnect(remoteEnd);
		
	}
	public void addSendListener(ByteBufferSendListener portSendListener) {
		sendNotfier.addSendListener(portSendListener);
	}
	public void removeSendListener(ByteBufferSendListener portSendListener) {
		sendNotfier.removeSendListener(portSendListener);
	}
	public void notifyPortSend(String aRemoteEnd, ByteBuffer message, int sendId) {
		sendNotfier.notifyPortSend(aRemoteEnd, message, sendId);
	}
	public SendTrapper<ByteBuffer, ByteBuffer> getSendTrapper() {
		return sendTrapper;
	}
	public void setSendTrapper(SendTrapper<ByteBuffer, ByteBuffer> newVal) {
		Tracer.info(this, "Set my send trapper to:" + newVal);
		sendTrapper = newVal;
	}
	
	
	@Override
	public void connected(String aRemoteEnd) {
		Tracer.info(this, "Received connected notificaton from driver");
		sendClientName();
		notifyConnect(aRemoteEnd);
		
	}
	@Override
	public void notConnected(String aRemoteEnd, String anExplanation) {
		Tracer.info(this, "Received not connected notification from driver");
		connected = false;
		notifyConnectFailure(aRemoteEnd, anExplanation);
	}
	@Override
	public void messageSent(String aRemoteEnd, ByteBuffer message,
			int sendId) {
		Tracer.info(this, "Received sent notification from driver");
		notifyPortSend(aRemoteEnd, message, sendId);
		
	}
	@Override
	public void  disconnected(String aRemoteEnd, boolean anExplicitDsconnection, String anExplanation) {
		Tracer.info(this, "Received disconnect notification from driver");
		connected = false;
		notifyDisconnect(aRemoteEnd, anExplicitDsconnection, anExplanation);
	}

	
}
