package inputport.datacomm.simplex.buffer;
import global.GlobalState;
import inputport.AConnectRegistrarAndNotifier;
import inputport.ConnectionListener;
import inputport.ConnectionRegistrarAndNotifier;
import inputport.datacomm.simplex.AReceiveRegistrarAndNotifier;
import inputport.datacomm.simplex.ReceiveListener;
import inputport.datacomm.simplex.ReceiveRegistrarAndNotifier;
import inputport.datacomm.simplex.ReceiveTrapper;

import java.nio.ByteBuffer;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import common.Tracer;



public class AGenericSimplexBufferServerInputPort<RequestChannelType, MessageChannelType>  implements GenericSimplexServerInputPort<RequestChannelType, MessageChannelType>   {
	protected ReceiveTrapper<ByteBuffer, ByteBuffer> receiveTrapper;
	boolean connected;
	
	ConnectionRegistrarAndNotifier connectNotifier = new AConnectRegistrarAndNotifier();
	ReceiveRegistrarAndNotifier receiptNotfier = new AReceiveRegistrarAndNotifier();
	SimplexBufferServerInputPortDriver<RequestChannelType, MessageChannelType> driver;

	String myName;
	String myId;
	protected Map<MessageChannelType, String> channelToClientName = new HashMap();
	protected Map<String, MessageChannelType> clientNameToChannel = new HashMap();
	public AGenericSimplexBufferServerInputPort (String aMyName, String anId) {
		myName = aMyName;
		myId = anId;
		receiveTrapper = GlobalState.getSimplexBufferServerIPTrapperSelector().createReceiveTrapper(this, receiptNotfier);
		Tracer.info(this, "Retrieved from global state receive trapper:" + receiveTrapper);
	}
	
	@Override
	public void connect() {
		if (connected) {
			Tracer.info(this, "Ignoring duplicate call to connect");
		}
		Tracer.info(this, "Changing connection status and asking driver to connect");
		connected = true;
		driver.connect();
	}
	protected void associate (MessageChannelType aChannelType, String aClientName) {
		Tracer.info(this, "Associating " + aClientName  + " with " + aChannelType);
		channelToClientName.put(aChannelType, aClientName);
		clientNameToChannel.put(aClientName, aChannelType);
	}
	@Override
	public void messageReceived(MessageChannelType aChannelType,
			ByteBuffer aMessage) {
		Tracer.info(this, "Received message:" + aMessage + " on channel:" + aChannelType);
		String clientName = channelToClientName.get(aChannelType);
		if (clientName == null) {
			int deducedLength = aMessage.limit() - aMessage.position();
			byte[] stringBytes = new byte[deducedLength];
			aMessage.get(stringBytes);
			clientName = new String(stringBytes);
			associate(aChannelType, clientName);
			Tracer.info(this, "ServerInputPort connected to:" + aChannelType);
			notifyConnect(clientName);
		} else {
			Tracer.info(this, "ServerInputPort received message " + aMessage + " from:" + aChannelType );
			notifyPortReceive(clientName, aMessage);			
		}		
	}
	@Override
	public void disconnect() {
		Tracer.info(this, "Disconnecting all message channels");
		for (MessageChannelType socketChannel: channelToClientName.keySet()) {
			String clientName = channelToClientName.get(socketChannel);
			driver.disconnect(socketChannel);
			notifyDisconnect(clientName, true, "Local process closed connection to destination");
		}

		channelToClientName.clear();
		clientNameToChannel.clear();

	}
	public void addConnectionListener(ConnectionListener portConnectListener) {
		connectNotifier.addConnectionListener(portConnectListener);
	}
	public void removeConnectionListener(ConnectionListener portConnectListener) {
		connectNotifier.removeConnectionListener(portConnectListener);
	}
	
	public void notifyConnect(String remoteEnd) {
		connectNotifier.notifyConnect(remoteEnd);
	}

	public void notifyDisconnect(String aRemoteEnd, boolean anExplicitDisconnection,
			String anExplanation) {
		connectNotifier.notifyDisconnect(aRemoteEnd, anExplicitDisconnection, anExplanation);

		
	}

	@Override
	public void disconnected(String aRemoteEndName,
			boolean anExplicitDisconnection, String anExplanation) {
		Tracer.info(this, "Received from channel disconnected message");
		notifyDisconnect(aRemoteEndName, anExplicitDisconnection, anExplanation);
		MessageChannelType channel = clientNameToChannel.get(aRemoteEndName);
		channelToClientName.remove(channel);
		channelToClientName.remove(aRemoteEndName);
	}
	public void notifyConnectFailure(String aRemoteEnd, String aMessage) {
		connectNotifier.notifyConnectFailure(aRemoteEnd, aMessage);
	}

	public void addReceiveListener(ReceiveListener<ByteBuffer> portReceiveListener) {
		receiptNotfier.addReceiveListener(portReceiveListener);
	}
	public void removeReceiveListener(ReceiveListener<ByteBuffer> portReceiveListener) {
		receiptNotfier.removeReceiveListener(portReceiveListener);
	}
	@Override
	public void notifyPortReceive(String remoteEnd, ByteBuffer message) {
		receiveTrapper.notifyPortReceive(remoteEnd, message);
	}
	@Override
	public void setDriver(SimplexBufferServerInputPortDriver<RequestChannelType, MessageChannelType> aDriver) {
		driver = aDriver;
	}
	@Override
	public String getLocalName() {
		return myName;
	}
	public ReceiveTrapper<ByteBuffer, ByteBuffer> getReceiveTrapper() {
		return receiveTrapper;
	}

	public void setReceiveTrapper(ReceiveTrapper<ByteBuffer, ByteBuffer> aReceiveTrapper) {
		Tracer.info(this, "Setting my receive trapper:" + aReceiveTrapper);
		this.receiveTrapper = aReceiveTrapper;
	}

	@Override
	public String getServerId() {
		return myId;
	}
	@Override
	public Set<String> getConnections() {
		Set<String> keySet = clientNameToChannel.keySet();
		Set<String> retVal = new HashSet();
		for (String name: keySet) {
			retVal.add(name);			
		}
		return retVal;
	}
	@Override
	public MessageChannelType getChannel(String clientName) {
		return  clientNameToChannel.get(clientName);		
	}
	@Override
	public String getClientName(MessageChannelType aChannel) {
		return  channelToClientName.get(aChannel);		
	}


	


	
	
}
