package inputport.datacomm.simplex.buffer.nio;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.channels.SelectableChannel;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;

import common.Tracer;


public class AConnectRequestResponse implements ConnectRequestResponse {
	InetAddress serverHost;
	SocketChannelConnectListener listener;
	int port;
	SocketChannel socketChannel;
	boolean connected;
	SelectionManager selectingRunnable;
	public AConnectRequestResponse(SelectionManager theSelectingRunnable, SocketChannel theSocketChannel, InetAddress theServerHost, int thePort, SocketChannelConnectListener theListener) {
		serverHost = theServerHost;
		listener = theListener;
		socketChannel = theSocketChannel;
		port = thePort;
		selectingRunnable = theSelectingRunnable;
	}	
	public InetAddress getServerHost() {
		return serverHost;
	}
	public int getPort() {
		return port;
	}
	@Override
	public SocketChannelConnectListener getListener() {
		return listener;
	}
	@Override
	public boolean makeRequest() {
		try {
//			Tracer.info(this, "Making request");
			// unless there is a gap between two connection requests, the second one fails with no warning
			// configuring the mode as blockng throws exception
			// only solution maybe to throw a sleep in here or in the program that calls connect.			
//			socketChannel.configureBlocking(false);
//			socketChannel.configureBlocking(true); // asynchrony is causing problems
			Tracer.info(this, "Making connect request for host " + serverHost + " port " + port);
			Tracer.info(this, "Registering connect op for:" + socketChannel );
			socketChannel.register(selectingRunnable.getSelector(), SelectionKey.OP_CONNECT);
			connected = socketChannel.connect(new InetSocketAddress(serverHost, port));
			Tracer.info(this, "Connection status for  " + socketChannel + " is " + connected);
			return connected;
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
	}
	@Override
	public void giveResponse() {
		try { 
//			Tracer.info(this, "Giving response for socket channel: " + socketChannel);
			connected = socketChannel.finishConnect();
//			Message.info("Finish connections status  for channel "  + socketChannel + " "  + connected);
			if (!connected) {
				Tracer.error("Could not finish connection for channel " + socketChannel);
			}
			SelectionKey selectionKey = socketChannel.keyFor(selectingRunnable.getSelector());		
//			connected = true;
			Tracer.info(this, "Registering read op for:" + socketChannel );
			selectionKey.interestOps(SelectionKey.OP_READ);
			if (listener != null)
				listener.connected(socketChannel);
		} catch (Exception e) {
			if (listener != null)
				listener.notConnected(socketChannel, e);
			else
				e.printStackTrace();
		}
	}
	@Override
	public SelectableChannel getChannel() {
		return socketChannel;
	}

}
