package inputport.datacomm.simplex.buffer.nio;

import java.io.EOFException;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.CancelledKeyException;
import java.nio.channels.SelectableChannel;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;
import java.nio.channels.spi.SelectorProvider;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ConcurrentLinkedQueue;

import common.Tracer;


public class ASelectionManager implements SelectionManager {
	public static final int MAX_BUFFERS = 10;
	public static final int READ_BUFFER_SIZE = 8192;
	Queue<RequestResponse> requestQueue = new ConcurrentLinkedQueue();	
	List<BufferedWrite> nextBufferedWrites = new ArrayList();
	Selector selector ;
	Map<SelectableChannel, Response> channelToResponse = new HashMap();
	Map<SelectableChannel, SocketChannelReadListener> channelToReadListener = new HashMap();
	Map<SelectableChannel, SocketChannelCloseListener> channelToCloseListener = new HashMap();
	// why not a single queue
	// why not selectable channel
	Map<SocketChannel, BlockingQueue<BufferedWrite>> channelToBufferedWriteBoundedBuffer = new Hashtable();	
	Map<SocketChannel, ByteBuffer> channelToReadBuffer = new HashMap();	
	
	public ASelectionManager () {
		try {
			selector = SelectorProvider.provider().openSelector();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	protected void allocateReadState(SocketChannel theSocketChannel) {
		ByteBuffer readBuffer = ByteBuffer.allocate(READ_BUFFER_SIZE);
		Tracer.info(this, "Allocating read buffer:" + readBuffer);
		readBuffer.mark();
		channelToReadBuffer.put(theSocketChannel, readBuffer);		
		
	}
	protected ByteBuffer getReadBuffer(SocketChannel theSocketChannel) {
		ByteBuffer readVal = channelToReadBuffer.get(theSocketChannel);
		if (readVal == null) 
			allocateReadState(theSocketChannel);
		return channelToReadBuffer.get(theSocketChannel);			
	}
	
	protected BlockingQueue<BufferedWrite> getBufferedWriteBoundedBuffer(BufferedWrite bufferedWrite) {
		SocketChannel channel = bufferedWrite.getSocketChannel();
		BlockingQueue<BufferedWrite> bufferedWriteBoundedBuffer = channelToBufferedWriteBoundedBuffer.get(channel);
		if (bufferedWriteBoundedBuffer == null) {
			bufferedWriteBoundedBuffer =  new ArrayBlockingQueue(MAX_BUFFERS);
			channelToBufferedWriteBoundedBuffer.put(channel, bufferedWriteBoundedBuffer);
		}
		return bufferedWriteBoundedBuffer;
	}
	
	public synchronized void putBufferedWrite (BufferedWrite bufferedWrite) {
		try {
			Tracer.info(this, "Started storing of buffered write with id:" + bufferedWrite.getId());
			BlockingQueue<BufferedWrite> bufferedWriteBoundedBuffer = getBufferedWriteBoundedBuffer(bufferedWrite);
			bufferedWriteBoundedBuffer.put(bufferedWrite);	
			if (registerWriteOpsForBufferedWrites(bufferedWrite)) {
				Tracer.info(this, "Waking up selector as new write registered");
				selector.wakeup();
			}
//			Tracer.info(this, "Finished storing of buffered write with id:" + bufferedWrite.getId());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}	
	public  synchronized void puRequestResponse (RequestResponse request) {	
		requestQueue.add(request);
		Tracer.info(this, "Added request:" + request);
	}		
	public Selector getSelector() {
		return selector;
	}
	void processRequestQueue() {
		for (Request request:requestQueue) {
			Tracer.info(this, "Calling makeRequest on:" + request);
			request.makeRequest(); // maybe do this only if connection status is false	
			Tracer.info(this, "Associating request  with channel to wait for response :" + request.getChannel());
			channelToResponse.put(request.getChannel(), (Response) request);			
		}
		requestQueue.clear();
	}

	protected void processBufferedWrites() {
		for (BlockingQueue<BufferedWrite> bufferedWriteBoundedBuffer: channelToBufferedWriteBoundedBuffer.values()) {
			processChannelBufferedWrites(bufferedWriteBoundedBuffer);

			
		}
	}
	boolean registerWriteOpsForBufferedWrites(BufferedWrite writeBuffer) {
		return registerWriteOpsForBufferedWrites(writeBuffer.getSocketChannel());
	}
//	@Override
	 boolean registerWriteOpsForBufferedWrites(SocketChannel channel) {
		BlockingQueue<BufferedWrite> bufferedWriteBoundedBuffer = channelToBufferedWriteBoundedBuffer.get(channel);

		if (!channel.isConnected() || 
		bufferedWriteBoundedBuffer.isEmpty()) {
			return false;
		}
		//if (key.interestOps() != SelectionKey.OP_WRITE)
		SelectionKey key = channel.keyFor(selector);
		try {
			Tracer.info(this, "Registering write op for:" + channel );
			key.interestOps(SelectionKey.OP_WRITE);
			return true;
		} catch (CancelledKeyException cke) { // we get this on socket close for some reason
			Tracer.info(this, "Received CancelledKeyException");
			close(key, cke);
			return false;
		}
		
	}
	 // keep getting concurrent change exception without synchronized
	synchronized void  registerWriteOpsForBufferedWrites() {

		for (SocketChannel channel: channelToBufferedWriteBoundedBuffer.keySet()) {
			if (!registerWriteOpsForBufferedWrites(channel))
				Tracer.info(this, "channel not connected or no pending writes:" + channel);
		}		
	}

	void  processChannelBufferedWrites(SelectionKey selectionKey) {	
		BlockingQueue<BufferedWrite> bufferedWriteBoundedBuffer = channelToBufferedWriteBoundedBuffer.get(selectionKey.channel());
		// could the bounded buffer be null?
		processChannelBufferedWrites(bufferedWriteBoundedBuffer);		
	}
	 void  processChannelBufferedWrites(BlockingQueue<BufferedWrite> bufferedWriteBoundedBuffer) {	
		for (BufferedWrite bufferedWrite: bufferedWriteBoundedBuffer) {
			if (bufferedWrite.doWrite())
				bufferedWriteBoundedBuffer.remove(bufferedWrite);
			else // why break out of inner loop
				break;
		}		
	}
	synchronized void processSelectedOperation() {
		for (SelectionKey selectionKey: selector.selectedKeys()) {
			if (!selectionKey.isValid()) break;
			if (selectionKey.isAcceptable() || selectionKey.isConnectable()) {
				Response response = channelToResponse.get(selectionKey.channel());
				Tracer.info(this, "Received connection accept/connect event on channel " + selectionKey.channel());	
				Tracer.info(this, "Calling giveResponse on:" + response);
				response.giveResponse();
//				Tracer.info(this, "Finished calling handler for accept.connect event");
				//selectionKey.interestOps(SelectionKey.OP_READ);
			}
			else if (selectionKey.isReadable()) {
				try {
//					Tracer.info(this, "Selector starts processing of read");
					processRead(selectionKey);
//					Tracer.info(this, "Selector finished processing of read");
				} catch (IOException e) {
					close(selectionKey, e);
				}
			}

			else if (selectionKey.isWritable()) {
				BlockingQueue<BufferedWrite> bufferedWriteBoundedBuffer = channelToBufferedWriteBoundedBuffer.get(selectionKey.channel());
				processChannelBufferedWrites(bufferedWriteBoundedBuffer);
				if (bufferedWriteBoundedBuffer.isEmpty()) {
					Tracer.info(this, "Selector registering read as no pending writes");
					selectionKey.interestOps(SelectionKey.OP_READ);
				}
			}					
		}
		selector.selectedKeys().clear();
		
	}
	@Override
	public void run() {		
		// executed after each wakeup event
		while (true) {
			try {
//				Message.info("Selector processing request queue");
				processRequestQueue();
//				Message.info("Selector registering write ops ");
				registerWriteOpsForBufferedWrites();
				// why not process the buffered writes
				//processBufferedWrites();
				Tracer.info(this, "Selector calls select");
				selector.select();
				Tracer.info(this, "Selector select unblocks");
				processSelectedOperation();			
			} catch (Exception e) {
				e.printStackTrace();
			}	
		}
	}
	void close(SelectionKey selectionKey, Exception readException)  {
		try {
			Tracer.info(this, "Closing and cancelling selection key for:" +  selectionKey.channel());
			selectionKey.channel().close();			
			selectionKey.cancel();
			notifyClose((SocketChannel) selectionKey.channel(), readException);
			return;
		} catch (IOException e) {
			e.printStackTrace();			
		}
	}
	protected int  readIntoBuffer(SocketChannel socketChannel) throws IOException {
	
		    ByteBuffer readBuffer = getReadBuffer(socketChannel);
			int bytesRead = socketChannel.read(readBuffer);
			Tracer.info(this, "Read: " + bytesRead + " bytes into buffer: " + readBuffer);
			if (bytesRead < 0)  {
				throw new EOFException("An existing connection was normally closed by the remote host.");				
			}
			return bytesRead;
				
	}
	protected void processRead(SelectionKey theSelectionKey) throws IOException {
		//try {
		SocketChannel socketChannel = (SocketChannel) theSelectionKey.channel();
		Tracer.info(this, "Processing read on channel:" + socketChannel);
		readIntoBuffer(socketChannel);
	    ByteBuffer readBuffer = getReadBuffer(socketChannel);
		readBuffer.flip();
		Tracer.info(this, "Read buffer after flip:" + readBuffer);
		ByteBuffer messageBuffer = ByteBuffer.wrap(readBuffer.array(), 0, readBuffer.remaining());
		notifyRead(socketChannel, messageBuffer, readBuffer.remaining());
		readBuffer.clear();	
		Tracer.info(this, "Cleared read buffer");
	}
	protected void notifyRead(SocketChannel theSocketChannel, ByteBuffer theBuffer, int length ) {
		SocketChannelReadListener readListener = channelToReadListener.get(theSocketChannel);
		if (readListener != null)
			readListener.socketChannelRead(theSocketChannel, theBuffer);
		
	}
	void notifyClose(SocketChannel theSocketChannel, Exception readException) {
		SocketChannelCloseListener closeListener = channelToCloseListener.get(theSocketChannel);
		if (closeListener != null)
			closeListener.closed(theSocketChannel, readException);	
		
	}
	
	@Override
	public void registerReadListener(SelectableChannel theChannel, SocketChannelReadListener theReadListener) {
		channelToReadListener.put(theChannel, theReadListener);		
	}
	@Override
	public void registerCloseListener(SelectableChannel theChannel, SocketChannelCloseListener theCloseListener) {
		channelToCloseListener.put(theChannel, theCloseListener);		
	}

}
