package inputport.datacomm.simplex.buffer.nio;

import inputport.datacomm.simplex.buffer.SimplexClientInputPortSkeleton;

import java.io.EOFException;
import java.io.IOException;
import java.net.InetAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;

import common.Tracer;

public class AnNIOSimplexBufferClientInputPortDriver  implements NIOSimplexClientInputDriver {
	InetAddress host;
	int port;
	protected SelectionManager selectingRunnable;
	protected String serverName;
	SocketChannel socketChannel;

	SimplexClientInputPortSkeleton<SocketChannel> clientInputPortSkeleton;
	public AnNIOSimplexBufferClientInputPortDriver(SelectionManager theSelectingRunnable, String theRemoteHostName, String theRemotePort, String aServerName) {
		try {
			host = InetAddress.getByName(theRemoteHostName);
			port = Integer.parseInt(theRemotePort);
			selectingRunnable = theSelectingRunnable;
			serverName = aServerName;
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	@Override
	public void setSkeleton(SimplexClientInputPortSkeleton<SocketChannel> theSkeleton) {
		clientInputPortSkeleton = theSkeleton;
	}
	@Override
	public SimplexClientInputPortSkeleton getSkeleton() {
		return clientInputPortSkeleton;
	}
	@Override
	public void connect() {
		socketChannel = createSocketChannel();
		registerWithSelectionRunnable();
	}
	void registerWithSelectionRunnable() {
		selectingRunnable.registerCloseListener(socketChannel, this);
		ConnectRequestResponse connectRequestResponse = 
			new AConnectRequestResponse(selectingRunnable, socketChannel, host, port, this);
		selectingRunnable.puRequestResponse(connectRequestResponse);
		Tracer.info(this, "Waking up selector to process newly queued connect request");
		selectingRunnable.getSelector().wakeup();
		Tracer.info(this, "ClientInputPort connected with channel:" + socketChannel);
	}
	SocketChannel createSocketChannel() {
		try {
			SocketChannel retVal = SocketChannel.open();
			retVal.configureBlocking(false);
			return retVal;

		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	@Override
	public void send(String destination, ByteBuffer message) {
		Tracer.info(this, "Sending messsage: " + message + " to " + socketChannel);
		BufferedWrite bufferedWrite = new ABufferedWrite(selectingRunnable, socketChannel, message, this, this);
		selectingRunnable.putBufferedWrite(bufferedWrite);
		
	}

	@Override
	public void connected(SocketChannel theSocketChannel) {

		clientInputPortSkeleton.connected(serverName);

		
	}
	@Override
	public void notConnected(
			SocketChannel theSocketChannel, Exception e) {
		clientInputPortSkeleton.notConnected (serverName, e.getMessage());		
	}
	@Override
	public void disconnect() {
		try {
			socketChannel.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	@Override
	public void closed(SocketChannel theSocketChannel,
			Exception anException) {
		clientInputPortSkeleton.disconnected(serverName, (anException instanceof EOFException), anException.getMessage());		
	}
	@Override
	public void written(SocketChannel socketChannel,
			ByteBuffer theWriteBuffer, int sendId) {
		clientInputPortSkeleton.messageSent(serverName, theWriteBuffer, sendId);		
	}

	
	
	
	
}
