package inputport.datacomm.simplex.buffer.nio;
import inputport.datacomm.simplex.buffer.SimplexServerInputPortSkeleton;

import java.io.EOFException;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;

import common.Tracer;



public class AnNIOSimplexBufferServerInpuPorttDriver implements NIOSimplexServerInputDriver  {
	SimplexServerInputPortSkeleton<ServerSocketChannel, SocketChannel> skeleton;
	protected SelectionManager selectingRunnable;
	int portNumber;
	ServerSocketChannel serverSocketChannel;	
	String serverId;
	//InetAddress localHost;
	public AnNIOSimplexBufferServerInpuPorttDriver (SelectionManager theSelectingRunnable, String thePortId) {
		selectingRunnable = theSelectingRunnable;
		portNumber = Integer.parseInt(thePortId);
//		myName = theMyName;	
	}
	@Override
	public void connect() {
		serverSocketChannel = createSocketChannel();
//		startChannel();
		if (serverSocketChannel != null)
			registerWithSelectionRunnable();
	}
	void registerWithSelectionRunnable() {		
		SocketChannelAcceptRequestResponse acceptRequestResponse = new AnAcceptRequestResponse(selectingRunnable, serverSocketChannel, this);
		selectingRunnable.puRequestResponse(acceptRequestResponse);	
		Tracer.info(this, "Waking up selector to process newly queued accept request");
		selectingRunnable.getSelector().wakeup();
	}
	ServerSocketChannel createSocketChannel() {
		try {			
			//localHost = InetAddress.getLocalHost();
			ServerSocketChannel retVal = ServerSocketChannel.open();
			retVal.configureBlocking(false);
			InetSocketAddress isa = new InetSocketAddress(portNumber);
			retVal.socket().bind(isa);
			return retVal;
		
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	
	@Override
	public void socketChannelAccepted(
			ServerSocketChannel theServerSocketChannel,
			SocketChannel theSocketChannel) {
		selectingRunnable.registerCloseListener(theSocketChannel, this);
		selectingRunnable.registerReadListener(theSocketChannel, this);
//		socketChannelToClientProxy.put(theSocketChannel, new AClientProxy(this));
			
	}

	@Override
	public void connected(SocketChannel theSocketChannel) {
		selectingRunnable.registerReadListener(theSocketChannel, this);
	}
		
	@Override
	public void notConnected(
			SocketChannel theSocketChannel, Exception e) {
		e.printStackTrace(); // this should not really happen
	}
	

	@Override
	public void closed(SocketChannel aSocketChannel,
			Exception aReadException) {
		skeleton.disconnected(skeleton.getClientName(aSocketChannel), aReadException instanceof EOFException, aReadException.getMessage());

	}

	@Override
	public void socketChannelRead(SocketChannel theSocketChannel,
			ByteBuffer theMessage) {
		skeleton.messageReceived(theSocketChannel, theMessage);

	}
	
	@Override
	public void setSkeleton(SimplexServerInputPortSkeleton<ServerSocketChannel, SocketChannel> theSkeleton) {
		skeleton = theSkeleton;
		
	}
	@Override
	public void disconnect(SocketChannel theChannel) {
		try {
			theChannel.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	@Override
	public void disconnect() {
		try {
			serverSocketChannel.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	
}
